/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.utils;

import java.util.Iterator;
import java.util.StringTokenizer;

import org.conqat.lib.commons.reflect.ReflectionUtils;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.model.IIdLabeled;
import org.fortiss.tooling.kernel.model.IIdLabeledReference;
import org.fortiss.tooling.kernel.model.INamedElement;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * Utility methods for accessing {@link IProjectRootElement}s.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: AA5BFF29ABFE9DEB170DB51BDB3B5427
 */
public final class ProjectRootElementUtils {

	/**
	 * Returns the instance of a project root element corresponding to the given
	 * class.
	 */
	public static <T extends IProjectRootElement> T getRootElement(
			EObject element, Class<T> clazz) {
		return ReflectionUtils.pickInstanceOf(clazz,
				IPersistencyService.INSTANCE.getTopLevelElementFor(element)
						.getRootModelElement().eContents());
	}

	/**
	 * Returns the model element with the given qualified name or
	 * <code>null</code> if no such element exists. The search is started from
	 * the given elements top-level parent.
	 */
	public static EObject findElementByName(String qualifiedName, EObject parent) {
		StringTokenizer tizer = new StringTokenizer(qualifiedName, "/");
		EObject current = IPersistencyService.INSTANCE.getTopLevelElementFor(
				parent).getRootModelElement();
		boolean found = true;
		while (found && tizer.hasMoreElements()) {
			String name = tizer.nextToken();
			found = false;
			for (EObject child : current.eContents()) {
				if (child instanceof INamedElement
						&& name.equals(((INamedElement) child).getName())) {
					current = child;
					found = true;
					break;
				}
			}
		}
		if (found) {
			return current;
		}
		return null;
	}

	/**
	 * Finds the model element referenced by the given
	 * {@link IIdLabeledReference} element or <code>null</code> if no such
	 * element exists.
	 */
	public static EObject findElementById(IIdLabeledReference reference) {
		return findElementById(reference.getIdReference(),
				IPersistencyService.INSTANCE.getTopLevelElementFor(reference)
						.getRootModelElement());

	}

	/**
	 * Finds the model element referenced by the given
	 * {@link IIdLabeledReference} element or <code>null</code> if no such
	 * element exists.
	 */
	public static EObject findElementById(int id, EObject root) {
		for (Iterator<EObject> iter = root.eAllContents(); iter.hasNext();) {
			EObject eo = iter.next();
			if (eo instanceof IIdLabeled && ((IIdLabeled) eo).getId() == id) {
				return eo;
			}
		}
		return null;
	}
}
